require "objc2swift_assistant/version"

require 'commander'
require_relative 'objc2swift_assistant/file_sets'
require 'objc2swift_assistant/recognizers/implementation_recognizer'
require 'objc2swift_assistant/recognizers/property_declaration_recognizer'
require 'objc2swift_assistant/recognizers/interface_recognizer'
require 'objc2swift_assistant/recognizers/category_recognizer'
require 'objc2swift_assistant/recognizers/method_recognizer'
require 'objc2swift_assistant/recognizers/enum_recognizer'
require 'objc2swift_assistant/recognizers/protocol_recognizer'
require 'objc2swift_assistant/recognizers/pragma_mark_recognizer'
require 'objc2swift_assistant/recognizers/at_sign_directives_recognizer'
require 'objc2swift_assistant/objc_2_swift_configuration'
require 'objc2swift_assistant/logging'

module Objc2swiftAssistant

  class Assistant
    attr_accessor :source_root
    attr_accessor :dest_root
    attr_accessor :recognizers
    attr_accessor :config_file_path
    attr_accessor :config
    attr_accessor :log_level_name
    attr_accessor :configuration
    attr_accessor :structure_option

    def initialize( source, dest, config_file, log_level_name, structure_option)
      @source_root = source
      @dest_root = dest
      @config_file_path = config_file.nil? ? nil : Pathname.new( config_file )
      @log_level_name = log_level_name
      @structure_option = structure_option
      @recognizers = [
          CategoryDeclarationRecognizer.new(),
          CategoryImplRecognizer.new(),
          ProtocolRecognizer.new(),
          #EnumRecognizer.new(),
          ImplementationRecognizer.new(),
          InterfaceRecognizer.new(),
          MethodRecognizer.new(),
          PropertyDeclarationRecognizer.new(),
          AtDirectiveRecognizer.new()
          #PragmaMarkRecognizer.new()
      ]
    end

    def update
      source_file_set = ObjCFileSet.new( @source_root, @config_hash )
      source_file_set.scan

    end

    def generate( dry_run )
      unless @config_file_path.nil?
        log_stage_start( 'Reading Configuration File' )
      end

      @recognizers.each do | recognizer |
        recognizer.configuration = @configuration
      end

      source_file_set = ObjCFileSet.new( @source_root, @recognizers, configuration:@configuration)

      log_stage_start( 'Scanning Objective-C Source Files' )
      source_file_set.scan

      log_stage_start( 'Recognizing code fragmants' )
      source_file_set.recognize_code_fragments

      log_stage_start( 'Converting to Swift' )
      source_file_set.prepare_conversion

      source_file_set.generate_swift_file_set( @dest_root, dry_run )

      source_file_set.dump( '', '   ', @configuration.dump_only_errors ) if @configuration.dump_source

      # TODO
      # source_file_set.generate_swift_files(dry_run)
      #
      # generator = SwiftFileGenerator( @dest_root, source_file_set )
      # generator.map_sources_to_destinations

      # if writeCode
      #
      # else

      # generator.concat_source_files()
    end

    def read_config_file
      if @config_file_path.nil?
        @configuration = ObjC2SwiftConfiguration.new( {}, @log_level_name, @structure_option )
      else
        file_data = @config_file_path.read
        config_hash = JSON.parse(file_data)
        @configuration = ObjC2SwiftConfiguration.new( config_hash, @log_level_name, @structure_option )
      end

      @configuration.load_configuration
    end

    def log_stage_start( text )
      return unless @verbose
      @configuration.log_verbose( "\n---------------------------------------- #{text} ----------------------------------------\n\n" )
    end

  end # class
end # module
