require "objc2swift_assistant/version"

require "objc2swift_assistant/objc_2_swift"

module Objc2swiftAssistant
  def de_comment_line( line )
    new_line = line
    m = line.match( /^(?<uncommented>.*)\/\/(?<comment>.*)/ )
    unless m.nil?
      new_line = m[ 'uncommented' ].strip
    end

    return new_line
  end
  module_function :de_comment_line

  def de_comment_lines( lines )
    # Line Comments
    return lines.map do |line|
      Objc2swiftAssistant::de_comment_line( line )
    end
  end
  module_function :de_comment_lines

  def is_getter_method_name( property_name, method_name )
    #TODO: add boolean rules
    return property_name == method_name
  end
  module_function :is_getter_method_name


  def is_setter_method_name( property_name, method_name )
    setter_name = "set" + property_name[0, 1].upcase + property_name[1..-1]
    return method_name == setter_name
  end
  module_function :is_setter_method_name

  class CodeRecognizer
    attr_accessor :for_root_entities
    attr_accessor :start_match_regex
    attr_accessor :match_region_class
    attr_accessor :source_file_type
    attr_accessor :match_region_class
    attr_accessor :voiding_match
    attr_accessor :configuration      # set after initialization

    def initialize( start_test_regex, match_region_class, source_file_type, for_root_entities, include_preceeding_comment=for_root_entities, voiding_regex: nil)
      @start_line_match_regex = start_test_regex
      @match_region_class = match_region_class
      @source_file_type = source_file_type
      @for_root_entities = for_root_entities
      @voiding_regex = voiding_regex
    end

    def matches( raw_file_lines )

      file_lines = Objc2swiftAssistant::de_comment_lines( raw_file_lines )

      matched_regions = []
      file_lines.each_with_index do |line, index|
        if @start_line_match_regex =~ line
          unless @voiding_regex.nil?
            if @voiding_regex =~ line
              break
            end
          end
          #print( line )
          matched_region = @match_region_class.new( index, @for_root_entities )
          matched_region.detection_line = line
          matched_region.configuration = @configuration
          matched_regions << matched_region
        end

      end

      @configuration.log_verbose( "" ) if matched_regions.length > 0
      matched_regions.each do |region|
        # if region.region_type == 'at_directive'
        @configuration.log_verbose( "#{region.region_type} #{region.detection_line}" )
        # end
      end

      return matched_regions
    end

    def should_scan_file( file_type  )
      return true
      # if @source_file_type == :all_source_files
      #   return [:header_file, :implementation_file].include?( file_type )
      # else
      #   file_type == @source_file_type
      # end
    end

  end


  class MigrationRegion < FailableProcessingElement

    attr_accessor :is_root_entity
    attr_accessor :allowed_parent_region_types
    attr_accessor :region_type
    attr_accessor :can_occur_in_class_decl

    attr_accessor :detection_line               # For diagnostics

    attr_accessor :region_identifier

    attr_accessor :is_single_line
    attr_accessor :starting_line_number         # actually a zero-based index
    attr_accessor :ending_line_number           # actually a zero-based index
    attr_accessor :ending_of_root_header        # i.e. (ideally) The end of the class ivar declaration
    attr_accessor :root_header                  # The content of the ivar declaration
    # attr_accessor :consumed_line_count
    attr_accessor :sub_regions
    attr_accessor :parent_region
    attr_accessor :configuration

    def initialize( starting_line_number, is_root_entity, region_type, can_occur_in_class_decl:false )
      @starting_line_number = starting_line_number
      @is_root_entity = is_root_entity
      @region_type = region_type
      @detection_line = detection_line
      @can_occur_in_class_decl = can_occur_in_class_decl
      @sub_regions = []
      @error_messages = []
      @is_single_line = false
    end

    def contains_line( line_number )
      line_number >= @starting_line_number && line_number <= @ending_line_number
    end

    def add_sub_region( sub_region )
      @sub_regions << sub_region
      sub_region.parent_region = self
    end

    def complete( file_lines )
      # Delimit sub-regions
      previous_sub_region = nil
      @sub_regions.each do |sub_region|
        resolve_ending_line_number( previous_sub_region, sub_region.starting_line_number - 1) unless previous_sub_region.nil?
        previous_sub_region = sub_region
      end
      resolve_ending_line_number( previous_sub_region, self.ending_line_number ) unless previous_sub_region.nil?

      self.extract_information( file_lines[ @starting_line_number..@ending_line_number] )
      @sub_regions.each do |sub_region|
        sub_region.complete( file_lines )
      end

      #Find the limits of the unrecognized code in the root migration region
      @ending_of_root_header = nil
      @sub_regions.each.with_index(0) do |sub_region, i|
        if ! sub_region.can_occur_in_class_decl
          @ending_of_root_header = sub_region.starting_line_number - 1
          break
        end
      end

      @ending_of_root_header = @ending_line_number if @ending_of_root_header.nil?
      @root_header = file_lines[ @starting_line_number..@ending_of_root_header ]

      # can_occur_in_class_decl
    end

    def resolve_ending_line_number( region, suggested_line_number )
      if region.is_single_line
        region.ending_line_number = region.starting_line_number
      else
        region.ending_line_number = suggested_line_number
      end
    end

    def extract_information( file_lines )
      # Do nothing by default
    end

    def generic_description( detail )
      return detail
    end

    def brief_description()
      return ""
    end

    def has_failed
      ! @error_messages.length > 0
    end

    # Output
    def dump( indent, tab, errors_only )
      dump_region_info( indent, tab, errors_only )
      @sub_regions.each do |sub_region|
        sub_region.dump( indent+tab, tab, errors_only )
      end
    end

    def dump_region_info( indent, tab, errors_only )
      # puts( indent + "#{@region_type} [lines: #{@starting_line_number || '?'}-#{@ending_line_number || '?'}]")
      printf( "%s%-25s [lines: %3d-%-3d] %s %s\n", indent, @region_type, @starting_line_number+1|| -1, @ending_line_number+1 || -1,
              brief_description(), @error_message.nil? ? '' : 'ERROR: ' + @error_message )
    end

  end

  class ClassRootRegion < MigrationRegion
    attr_accessor :class_name

    def initialize( starting_line_number, is_root_entity, region_type_name )
      super( starting_line_number, is_root_entity, region_type_name )
    end

    def description()
      generic_description( "Class: #{@class_name}" )
    end
  end

end