require_relative "version"

module Objc2swiftAssistant

  SUBDIR_KEY = 'subdirs'
  PATH_KEY = 'path'


  class FileHierarchicalConfig
    attr_accessor :node_class
    attr_accessor :config_hash
    attr_accessor :configs_by_path
    attr_accessor :failure_reason
    attr_accessor :all_valid_keys
    attr_accessor :failure_reason


    def initialize( config_hash, config_keys )
      @node_class = FileHierarchicalConfigNode
      @all_valid_keys = config_keys + [ SUBDIR_KEY, PATH_KEY ]
      @config_hash = config_hash
      @configs_by_path = {}
    end

    def load_configuration
      # path test
      # p = Pathname.new( "/a/b/c/d")
      # done = false
      # until done
      #   puts( "path= #{p.to_s}")
      #   p = p.parent
      # end

      # @failure_reason = 'No Root configuration specified' unless root_hash.path.to_s == '' TODO: Fail is path specified
      root_node = node_class.new( self, @config_hash, '.' )
      add_config_node( root_node )

      # root_node.apply_wildcards
    end

    def add_config_node( config_node )
      if @configs_by_path.has_key?( config_node.path_from_root.to_s )
        return false, "A configuration node already exists for #{config_node.path_from_root.to_s}"
      end

      @configs_by_path[ config_node.path_from_root.to_s ] = config_node
      return true, nil
    end

    def config_value( path_str, key )
      pathname = Pathname.new( path_str )
      value = nil
      config = nil
      while config.nil?
        config = @configs_by_path[ pathname.to_s ]
        break if pathname.to_s == "." && config.nil?    # This should not happen, but just in case
        pathname = pathname.parent
      end

      value = config.config_value_for_key( key, path_str ) unless config.nil?
      value
    end

    def config_value_defaulted( path_str, key, default )
      value = config_value( path_str, key )
      value.nil? ? default : value
    end
  end

  class FileHierarchicalConfigNode
    attr_accessor :configuration
    attr_accessor :path_from_parent
    attr_accessor :path_from_root
    attr_accessor :parent_node
    attr_accessor :child_nodes
    attr_accessor :wildcard_nodes
    attr_accessor :configuration_hash


    def initialize( configuration, node_hash, relative_path, parent:nil, is_wildcard:false )
      @configuration = configuration
      @configuration_hash = node_hash || {}
      @path_from_parent = Pathname.new( relative_path )    #'.' is the "root" of relative Pathname. i.e. Pathname.parent.parent...
      @child_nodes = []

      if( parent.nil? )
        @path_from_root = Pathname.new( relative_path ) unless is_wildcard    # Parents set on clones of this object
        @parent_node = nil
      else
        add_to_parent( parent )
      end

      @wildcard_nodes = []

      child_hashes = @configuration_hash[ "subdirs" ] || nil
      unless child_hashes.nil?
        child_hashes.each do |child_hash|
          path_str = child_hash['path']
          make_child_node( path_str, child_hash )
        end
      end

      post_process_config
    end

    def make_child_node( path_str, child_hash )
      m = path_str.match(/\*\*\/(?<path>.*)/)
      if m.nil?
        child = @configuration.node_class.new( @configuration, child_hash, path_str, parent: self )
        success, failure_reason = configuration.add_config_node(child)
        @configuration.log_error("Could not add child configuration for path:#{child.path} to parent node with path:#{@path} - #{failure_reason})") unless success
      else
        path_str = m[ 'path' ]
        wild_child = @configuration.node_class.new(@configuration, child_hash, path_str, is_wildcard:true)
        @wildcard_nodes << wild_child
      end
    end


    def add_to_parent( parent )
      @parent_node = parent
      @parent_node.child_nodes << self
      @path_from_root = parent.path_from_root.join( @path_from_parent )
    end

    def post_process_config()
      # Hook for subclasses
    end

    # def apply_wildcards( parent_wildcards=[] )
    #   all_wildcards = @wildcard_nodes + parent_wildcards
    #   @child_nodes.clone.each { |node| node.apply_wildcards( all_wildcards ) }
    #
    #   all_wildcards.each do |wildcard_node|
    #     new_child = wildcard_node.clone
    #     new_child.add_to_parent( self )
    #     success, failure_reason = configuration.add_config_node( new_child )    #todo look for errors
    #   end
    # end

    def config_value_for_key( key, path_str )
      value = nil
      unless path_str.nil? || @wildcard_nodes.length == 0
        catch :wildcard_value_found do
          @wildcard_nodes.each do |wildcard_node|
            pathname = Pathname.new( path_str )
            until pathname.to_s == @path_from_root.to_s
              if pathname.to_s.end_with?( wildcard_node.path_from_parent.to_s )
                value = wildcard_node.config_value_for_key( key, nil )
                throw :wildcard_value_found
              end
              pathname = pathname.parent
            end
          end
        end
      end

      if value.nil?
        value = @configuration_hash[ key ]
        value ||= parent_node.config_value_for_key( key, path_str ) unless parent_node.nil?
      end

      value
    end

  end

end
