require_relative 'version'
require_relative 'objc_2_swift_type_mapping'
require_relative 'objc_variable_types'

module Objc2swiftAssistant

  class Objc2SwiftBlockConverter
    attr_accessor :type_converter

    def initialize( type_converter )
      @type_converter = type_converter
    end

    def block_sig_for_method_arg( arg_type_str )
      sig = Objc2SwiftBlockSignature.new( self )
      sig.from_argument_type( arg_type_str)
      sig
    end

    def block_sig_for_property( arg_type_str )
      sig = Objc2SwiftBlockSignature.new( self )
      sig.from_property_type( arg_type_str)
      sig
    end

    def block_sig_for_components( return_type_str, *arg_strings )
      sig = Objc2SwiftBlockSignature.new( self )
      sig.from_components( return_type_str, arg_strings )
      sig
    end
  end

  class Objc2SwiftBlockSignature
    attr_accessor :block_converter
    attr_accessor :return_type
    attr_accessor :arguments

    def initialize( block_converter )
      @block_converter = block_converter
    end

    def from_components( return_type_str, arg_strings )
      @return_type = @block_converter.type_converter.swift_type_for_objc_type( return_type_str )

      @arguments = arg_strings.map do |arg_str|
        arg_for_objc_str( arg_str.strip )
      end
    end

    def from_argument_type( arg_type )
      if arg_type.count('^') > 1
        failure = "Cannot convert nested Block: #{arg_type}"
        return "String /*#{failure}*/", failure
      else
        m = arg_type.match( /(?<return_type>[^\(]*)\s*\(\s*\^\s*\)\s*\((?<arg_string>.*)\)/ )
        if m.nil?
          failure = "Cannot match block: #{arg_type}"
          return "String /*#{failure}*/", failure
        else
          objc_return_type = m[ 'return_type' ].strip!
          @return_type = @block_converter.type_converter.swift_type_for_objc_type( objc_return_type )

          arg_string = m[ 'arg_string' ]
          arg_strings = arg_string.nil? ? [] : arg_string.split( ',' )

          @arguments = arg_strings.map do |arg_str|
            arg_for_objc_str( arg_str.strip )
          end
        end
      end
    end

    def swift_declaration( full=true )
      swift_args = @arguments.map do |arg|
        if arg.param_name.nil?
          str = "#{arg.param_type}"
        else
          str = "#{arg.param_name}: #{arg.param_type}"
        end
        str
      end

      sig = "(( #{swift_args.join( ', ' )} )"
      if full || !return_type.nil?
        if return_type.nil?
          sig += " -> ())" if return_type.nil?
        else
          sig += " -> #{return_type})"
        end
      else
        sig += " -> #{return_type})" unless return_type.nil?
      end
      sig
    end

    def from_property_type( prop_type )

    end

    def arg_for_objc_str( arg_str )
      argument = ObjCBlockParameter.new()
      argument.from_declaration( arg_str )
      argument
    end
  end
end
