require_relative "version"
require_relative "file_hierarchical_config"
require_relative "objc_2_swift_type_mapping"
require_relative "objc_2_swift_block_conversion"

module Objc2swiftAssistant

  LOG_LEVEL_NONE        = 0
  LOG_LEVEL_ERRORS      = 1
  LOG_LEVEL_WARNINGS    = 2
  LOG_LEVEL_VERBOSE     = 3
  LOG_LEVEL_DEBUG       = 4

  LOG_LEVELS_BY_NAME = {
      "error" => LOG_LEVEL_ERRORS,
      "warning" => LOG_LEVEL_WARNINGS ,
      "verbose" => LOG_LEVEL_VERBOSE ,
      "debug" => LOG_LEVEL_DEBUG,
      "none" => LOG_LEVEL_NONE
  }


  COMPANY_NAME_KEY                = 'company_name'
  OMIT_FILE_KEY                   = 'omit_file'
  EMIT_ORIGINAL_SIGNATURES_KEY    = 'emit_original_signatures'
  EMIT_ORIGINAL_BODIES_KEY        = 'emit_original_bodies'
  EMIT_UNCONVERTED_CONTENT_KEY    = 'emit_unconverted_content'

  ALL_CONFIG_KEYS = [
      COMPANY_NAME_KEY,
      OMIT_FILE_KEY,
      EMIT_ORIGINAL_SIGNATURES_KEY,
      EMIT_ORIGINAL_BODIES_KEY,
      EMIT_UNCONVERTED_CONTENT_KEY
  ]

  class ObjC2SwiftConfiguration < FileHierarchicalConfig
    attr_accessor :type_mapper
    attr_accessor :block_converter
    attr_accessor :structure_option

    attr_accessor :log_level


    def initialize( config_hash, log_level_name, structure_option )
      super( config_hash, ALL_CONFIG_KEYS )
      @node_class = ObjC2SwiftConfigNode
      @type_mapper = Objc2SwiftTypeMapper.new()
      @block_converter = Objc2SwiftBlockConverter.new( @type_mapper )
      @log_level = LOG_LEVELS_BY_NAME[ log_level_name ]
      @structure_option = structure_option
    end

    def omit_file( file_path )
      # val = config_value(file_path, OMIT_FILE_KEY)
      # puts( "Omit File: #{file_path} #{val ? 'true' : 'false' }" )  # nil will evaluate to false
      config_value( file_path, OMIT_FILE_KEY )

    end

    def company_name( file_path )
      config_value( file_path, COMPANY_NAME_KEY )
    end

    def emit_original_signatures( file_path )
      config_value_defaulted( file_path, EMIT_ORIGINAL_SIGNATURES_KEY, true )
    end

    def emit_original_bodies( file_path )
      config_value_defaulted( file_path, EMIT_ORIGINAL_BODIES_KEY, true )
    end

    def emit_unconverted_content( file_path )
      config_value_defaulted( file_path, EMIT_UNCONVERTED_CONTENT_KEY, true )
    end

    # Logging
    def log_verbose( str )
      puts( str ) if @log_level >= LOG_LEVEL_VERBOSE
    end

    def log_warning( str )
      puts( "WARNING: " + str ) if @log_level >= LOG_LEVEL_WARNINGS
    end

    def dump_source
      [ 'all', 'errors' ].include?( @structure_option )
    end

    def dump_only_errors
      @structure_option == 'errors'
    end

  end

  class ObjC2SwiftConfigNode <  FileHierarchicalConfigNode

    def post_process_config ()
      ignore_file_paths = @configuration_hash[ "ignore" ] || []
      ignore_file_hash = { OMIT_FILE_KEY => true }
      ignore_file_paths.each do | path_str |
        make_child_node( path_str, ignore_file_hash )
      end
    end

  end

end
