require_relative "../version"

require_relative "../code_recognizer"
require_relative "../objc_variable_types"

module Objc2swiftAssistant

  class MethodRecognizer < CodeRecognizer
    def initialize( )
      super( /^\s*[-+]\s*\(/, MethodRegion, :implementation, false )
    end
  end


  class MethodRegion < MigrationRegion
    # ObjC methods do not have "names"
    attr_accessor :objc_signature
    attr_accessor :is_class_method
    attr_accessor :return_type
    attr_accessor :return_pointer_level
    attr_accessor :return_nillable_qualifier
    attr_accessor :raw_parameter_text
    attr_accessor :parameters
    attr_accessor :takes_arguments
    attr_accessor :all_lines
    attr_accessor :unmatched_lines

    def initialize(starting_line_number, is_root_entity )
      super(starting_line_number, is_root_entity, METHOD_INDETERMINATE_KEY )
      @unmatched_lines = []
      @all_lines = []
    end

    def extract_information( file_slice )
      @all_lines = file_slice
      method_text = file_slice.join( "\n" )
      m = method_text.match( /^\s*(?<instance_or_class>[+-])\s*\((?<return_type>\w*)\s*(?<return_pointer>\*+)?\s*(?<nil_return_qualifier>(__nullable|__nonnull))?\s*\)(?<name_and_params>[^\{;]*)\s*(?<declaration_or_implementation>(\{|(;\s*\{)+|;))/m )
      if m.nil?
        @configuration.log_warning( "Could not match method (implemetation or declaration) in #{file_slice[0]}..." )
      else
        @is_class_method = m['instance_or_class'] == '+'
        @return_nillable_qualifier = m[ 'nil_return_qualifier' ]
        @return_type = m[ 'return_type' ]
        @return_pointer_level = m[ 'return_pointer' ].nil? ? 0 : m[ 'return_pointer' ].length
        @raw_parameter_text = m[ 'name_and_params' ]
        @parameters, @takes_arguments = parse_parameters( @raw_parameter_text )
        @objc_signature = make_signature
        @unmatched_lines = m.post_match.lines
        is_declaration = m['declaration_or_implementation'] == ';'
        @region_type = is_declaration ? METHOD_DECLARATION_KEY : METHOD_IMPLEMENTATION_KEY
        # log_verbose( description )
      end
    end

    def parse_parameters( parameter_text )
      takes_arguments = true
      parameters = []
      matches = []

      # handle single mthod names
      if parameter_text.include?( ':' )
        # Split the arguments including (simple) block arguments
        parameter_text.scan( /(?<label>\w*)\s*:\s*\((?<is_weak>__weak)?(?<value>[^:]*)\)\s*(?<arg>[^:\s]*)/ ){ matches << $~ }
        if matches.length > 0
          # puts( matches)
          matches.each do |m|
            param = ObjCMethodParameter.new()
            param.param_label = m[ 'label' ].strip
            param.param_name = m[ 'arg' ].strip
            unless m[ 'is_weak' ].nil?
              param.is_weak = true
            end
            value =  m[ 'value' ].strip
            if value.include? '^'
              param.is_block_type = true    # Its a block parameter
              param.param_type = value
            else
              param.is_block_type = false
              m = value.match( /(?<null_qualifier>(__nullable|__nonnull))?\s*(?<type>\w*)\s*(?<pointer>\*+)?/ )
              if m.nil?
                param.match_failure = "Could not match #{value} as a non-blocl parameter"
              else
                param.param_type = m[ 'type' ].strip
                param.null_qualifier = m[ 'null_qualifier' ]
                param.pointer_level = m[ 'pointer' ].nil? ? 0 : m[ 'pointer' ].length
              end
            end

            if param.param_label.nil?
              param.param_label = '/* Unknown */'
            end

            parameters << param
            takes_arguments = true
          end
        end
      else
        param = ObjCMethodParameter.new()
        param.param_label = parameter_text.strip    # No arguments to method: create a parameter with only a label
        parameters << param
        takes_arguments = false
      end

      return parameters, takes_arguments
    end

    # def extract_information( file_slice )
    #   method_text = file_slice.join( "\n" )
    #   m = method_text.match( /^\s*(?<instance_or_class>[+-])\s*\((?<return_type>\w*)\s*(?<return_pointer>\*+)?\)(?<name_and_params>[^\{;]*)\s*(?<declaration_or_implementation>(\{|(;\s*\{)+|;))/m )
    #   if m.nil?
    #     puts( "WARNING: Could not match method (implemetation or declaration) in #{file_slice[0]}..." )
    #   else
    #     @is_class_method = m['instance_or_class'] == '+'
    #     @return_type = m[ 'return_type' ]
    #     @return_pointer_level = m[ 'return_pointer' ].nil? ? 0 : m[ 'return_pointer' ].length
    #     @raw_parameter_text = m[ 'name_and_params' ]
    #     @parameters, @takes_arguments = parse_parameters( @raw_parameter_text )
    #     @objc_signature = make_signature
    #     @unmatched_lines = m.post_match.lines
    #     is_declaration = m['declaration_or_implementation'] == ';'
    #     @region_type = is_declaration ? METHOD_DECLARATION_KEY : METHOD_IMPLEMENTATION_KEY
    #     puts( description )
    #   end
    # end
    #
    # def parse_parameters( parameter_text )
    #   takes_arguments = true
    #   parameters = []
    #   matches = []
    #   parameter_text.scan( /(?<label>\w*)\s*:\s*\(\s*(?<null_qualifier>(__nullable|__nonnull))?\s*(?<type>\w*)\s*(?<pointer>\*+)?\s*\)\s*(?<arg>\w*)/ ){ matches << $~ }
    #   if matches.length > 0
    #     puts( matches)
    #     matches.each do |m|
    #       param = MethodParameter.new()
    #       param.param_label = m[ 'label' ]
    #       param.param_name = m[ 'arg' ]
    #       param.param_type = m[ 'type' ]
    #       param.null_qualifier = m[ 'null_qualifier' ]
    #       param.pointer_level = m[ 'pointer' ].nil? ? 0 : m[ 'pointer' ].length
    #       parameters << param
    #       takes_arguments = true
    #     end
    #   else
    #     param = MethodParameter.new()
    #     param.param_label = parameter_text    # No arguments to method: create a parameter with only a label
    #     parameters << param
    #     takes_arguments = false
    #   end
    #
    #   return parameters, takes_arguments
    # end


    def make_signature()
      signature = ''
      if @takes_arguments
        self.parameters.each do |param|
          signature << "#{param.param_label}:"
        end
      else
        signature = self.parameters[0].param_label
      end

      return signature.strip
    end

    def takes_arguments
      return ! @parameters[0].param_name.nil?
    end

    def params_descr()
      'NYI'
    end

    def description()
        return generic_description( "#{'+' if @is_class_method} return_type:#{@return_type} name:#{@name} #{'declaration' if @is_declaration} #{"body_lines:#{@unmatched_lines.length}" if @unmatched_lines.length > 0}" )
    end

    def brief_description
      # plus_minus = @is_class_method ? '+' : '-'
      # pointerText = ""
      # pointerText = " *****"[1..@pointer_level+1] unless @pointer_level.nil? || @pointer_level == 0
      # return "#{plus_minus} (#{@return_type}#{pointerText}) #{@raw_parameter_text}"

      return @objc_signature
    end

  end

end